<?php

class LA_Testimonials_App_ActivatorFree
{
    protected static $plugin;

    public function __construct($plugin)
    {
        $this->define_static_vars($plugin);
    }

    /**
     * Define static variables
     *
     * @param $plugin
     */
    protected function define_static_vars($plugin)
    {
        foreach ($plugin as $key => $value) {
            self::$plugin[$key] = $value;
        }
    }

    /**
     * Display error message after activation fail
     */
    public static function admin_message()
    {
        $message = sprintf(
            __('Plugin <b>%1$s</b> require PHP version %2$s or higher.'),
            self::$args['label'],
            self::$args['require_php']
        );
        $class = "error";
        echo "<div class=\"$class\"> <p>$message</p></div>";
    }

    /**
     * Activate plugin if PHP version compatible
     */
    public static function activate()
    {
        if (!self::compatible_version()) {
            add_action('admin_notices', 'admin_message');
            deactivate_plugins(self::$plugin['name']);
        }
    }

    /**
     * Compare PHP versions
     *
     * @return bool
     */
    public static function compatible_version()
    {
        if (version_compare(PHP_VERSION, self::$plugin['require_php']) == -1) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * If plugin already enabled in some way
     *
     * Check PHP version to prevent WP admin crash and deactivate plugin
     */
    public static function check_version()
    {
        if (!self::compatible_version()) {
            $plugin = self::$plugin['name'].'/init.php';
            if (is_plugin_active($plugin)) {
                deactivate_plugins($plugin);
            }
            add_action('admin_notices', 'LA_Testimonials_App_ActivatorFree::admin_message');
            if (isset($_GET['activate'])) {
                unset($_GET['activate']);
            }
        }
    }
}