<?php
namespace ExcitedFree\App;

use DrewM\MailChimp\MailChimp;
use ExcitedFree\App\Model\Testimonial;
use ExcitedFree\App\Model\Group;
use ExcitedFree\App\Template;
use ExcitedFree\App\Shortcode;
use LooksAwesome\Common\Validator;
use LooksAwesome\Common\Response;
use LooksAwesome\Common\Editor;
use LooksAwesome\Common\Post;
use ReCaptcha\ReCaptcha;

class Ajax{
    protected $response;
    protected static $namespace;

    public function __construct(){
        $this->response = new \WP_Ajax_Response;
        self::$namespace = \LA_Testimonials_Free::$plugin['namespace'];
    }

    /**
     * Get Model data by ID
     */
    public function ajax_handle_get_testimonial(){
        $input = $_REQUEST['data'];
        $post = Testimonial::find($input['id']);

        $thumb_id = get_post_thumbnail_id($post->ID);
        $thumb_url = wp_get_attachment_image_src($thumb_id, 'a-thumbnail');

        $terms = wp_get_post_terms($post->ID, Testimonial::getTaxonomy(), array('fields' => 'names'));

        $defaults = Testimonial::getDefaults();
        $post = $this->prepare_testimonial($post, $defaults);

        $result = array(
            'model' => 'testimonial',
            'id' => $post->ID,
            'name' => $post->post_title,
            'category' => $terms,
            'short_text' => $post->post_excerpt,
            'full_text' => $post->post_content,
            'post_status' => $post->post_status,
            'foto' => $thumb_url[0] ?: '', // $thumb_url returns array (url, width, height, (bool) resized)
            'fill_rules' => Testimonial::$fill_rules,
            'custom_fields' => array(
                'thumbnail_id' => $thumb_id, // $thumb id
            )
        );

        // mass assignment for custom fields
        foreach($defaults['custom_fields'] as $key => $value){
            $result['custom_fields'][$key] = isset($post->$key) ? $post->$key : $post->custom_fields[$key];
        }

        $response = Response::get($this->response, json_encode($result), 'get_model');
        $response->send();

        exit();
    }

    protected static function prepare_testimonial($post, $defaults){
	
        foreach($defaults as $key => $value){
            if(is_array($value)){
                $post->$key = self::prepare_testimonial($post, $defaults[$key]);
            }else{
                $post->$key = isset($post->$key) ? $post->$key : $defaults[$key];
            }
        }
        return $post;
    }

    protected function make_testimonial($post){
        // mass assignment for custom fields
        $custom_fields = $post->ID == 0 ? $post->custom_fields : get_post_custom($post->ID);
        foreach($custom_fields as $key => $value){
            $result['custom_fields'][$key] = isset($post->$key) ? $post->$key : $post->custom_fields[$key];
        }

        $result['model'] = 'testimonial';
        $result['id'] = $post->ID;
        $result['name'] = $post->post_title;
        $result['short_text'] = $post->post_excerpt;
        $result['fill_rules'] = Testimonial::$fill_rules;

        return $result;
    }

    protected function make_group($post){
        // get group testimonials list
     
		$testimonials = $post->testimonials;
        $shortcode = new Shortcode();
        $post = Group::setDefaults($post);
        
		if(gettype($testimonials) == 'array' && count($testimonials) > 0){
		
            $testimonials = Testimonial::where(
                array(
                    'post_type' => Testimonial::$post_type,
                    'post__in' => $testimonials,
                    'orderby' => 'post__in',
                    'posts_per_page' => -1
                )
            );
            foreach($testimonials as $testimonial){
                $testimonial->post_id = $testimonial->ID;
                $foto = wp_get_attachment_image_src(
                    get_post_thumbnail_id($testimonial->ID),
                    'a-thumbnail'
                );
                $testimonial->foto = $foto[0];
                $testimonial->position = get_post_meta($testimonial->ID, 'position', true);

                $testimonial->rating = get_post_meta($testimonial->ID, 'rating', true);
                $groups = get_post_meta($testimonial->ID, 'groups', true);
                $testimonial->groups_count = count($groups);
                // $testimonial->group_name = '';
                if($testimonial->post_status == 'pending'){
                    $group = Group::find($groups[0]);
                    $testimonial->group_name = strlen($group->post_title) > 20 ? substr($group->post_title, 0 , 20) . '...' : $group->post_title;
                }
            }
        }

        // mass assignment for custom fields
//        $custom_fields = $post->ID == 0 ? $post->custom_fields : get_post_custom($post->ID);
        $group_defaults = Group::getDefaults();
        $custom_fields = $post->ID == 0 ? $post->custom_fields : $group_defaults['custom_fields'];
        foreach($custom_fields as $key => $value){
            //$result[$key] = isset($post->$key) ? $post->$key : $post->custom_fields[$key];
            $result[$key] = isset($post->$key) ? $post->$key : $custom_fields[$key];
        }

        $result['model'] = 'group';
        $result['id'] = $post->ID;
        $result['name'] = $post->post_title;
        $result['description'] = $post->post_excerpt;
        $result['testimonials'] = $testimonials;
        $result['fill_rules'] = Group::$fill_rules;

        // backward compatibility for old default structure
        $result['template'] = $post->template ?: $post->styles[$post->layout]['template'];
        if(isset($custom_fields['template'])){
            $result['template'] = $custom_fields['template'] ?: $post->styles[$post->layout]['template'];
        }

        foreach(Group::$layouts as $k => $v){
            $result[$k . '_html'] = $shortcode->preview($post, $k);
        }

        return $result;
    }

    /**
     * Get Model data by ID
     */
    public function ajax_handle_get_group(){
        $input = $_REQUEST['data'];
        $post = Group::find($input['id']);

        $result = $this->make_group($post);			

        $response = Response::get($this->response, json_encode($result), 'get_model');
		
        $response->send();

        wp_die();
    }

    /**
     * Get Testimonial defaults
     */
    public function ajax_handle_get_testimonial_defaults(){
        $defaults = Testimonial::getDefaults();

        $post = new \stdClass;
        $post->ID = 0;
        $post->post_name = '';
        $post->post_type = 'testimonial';
        $post->post_excerpt = '';
        $post->custom_fields = $defaults['custom_fields'];

        $post = new \WP_Post($post);
        $result = $this->make_testimonial($post);

        $response = Response::get($this->response, json_encode($result), 'get_testimonial_defaults');
        $response->send();

        wp_die();
    }

    /**
     * Get Group defaults
     */
    public function ajax_handle_get_group_defaults(){
        $defaults = Group::getDefaults();

        $post = new \stdClass;
        $post->ID = 0;
        $post->post_name = '';
        $post->post_type = 'group';
        $post->post_excerpt = '';
        $post->testimonials = array();
        $post->custom_fields = $defaults['custom_fields'];

        $post = new \WP_Post($post);
        $result = $this->make_group($post);

        $response = Response::get($this->response, json_encode($result), 'get_group_defaults');
        $response->send();

        wp_die();
    }

    protected function avoid_empty_field($input, $key){
        return isset($input[$key]) ? $input[$key] : '';
    }

    /**
     * Route for add Testimonial Ajax request
     *
     * @return xml WP_Response
     */
    public function ajax_handle_update_or_new_testimonial(){
        if(!empty($_POST['data'])){
            $input = $_REQUEST['data']['data'];
        }
        $cf = isset($input['custom_fields']) ? $input['custom_fields'] : '';

        $input['id'] = $this->avoid_empty_field($input, 'id');
        $input['name'] = $this->avoid_empty_field($input, 'name');
        $input['category'] = $this->avoid_empty_field($input, 'category');
        $input['short_text'] = $this->avoid_empty_field($input, 'short_text');
        $input['full_text'] = $this->avoid_empty_field($input, 'full_text');
        $input['custom_fields'] = array(
            'created_from' => $this->avoid_empty_field($cf, 'created_from'),
            'position' => $this->avoid_empty_field($cf, 'position'),
            'thumbnail_id' => $this->avoid_empty_field($cf, 'thumbnail_id'),
            'rating' => $this->avoid_empty_field($cf, 'rating'),
            'rating_label' => $this->avoid_empty_field($cf, 'rating_label'),
            'source_name' => $this->avoid_empty_field($cf, 'source_name'),
            'source_url' => $this->avoid_empty_field($cf, 'source_url'),
            'custom_url' => $this->avoid_empty_field($cf, 'custom_url'),
            'email' => $this->avoid_empty_field($cf, 'email'),
            'phone' => $this->avoid_empty_field($cf, 'phone'),
            'skype' => $this->avoid_empty_field($cf, 'skype'),
            'location' => $this->avoid_empty_field($cf, 'location'),
            'facebook' => $this->avoid_empty_field($cf, 'facebook'),
            'twitter' => $this->avoid_empty_field($cf, 'twitter'),
            'linkedin' => $this->avoid_empty_field($cf, 'linkedin'),
            'google' => $this->avoid_empty_field($cf, 'google'),
            'instagram' => $this->avoid_empty_field($cf, 'instagram'),
            'pinterest' => $this->avoid_empty_field($cf, 'pinterest'),
            'youtube' => $this->avoid_empty_field($cf, 'youtube'),
            'vimeo' => $this->avoid_empty_field($cf, 'vimeo'),
            'dribble' => $this->avoid_empty_field($cf, 'dribble'),
            'flickr' => $this->avoid_empty_field($cf, 'flickr'),
            'github' => $this->avoid_empty_field($cf, 'github'),
        );

        $validator = new Validator(Testimonial::$model_name, self::$namespace);
        $validation = $validator->validate($input);

        if(count($validation->errors) > 0){
            $response = Response::get($this->response, $validation, 'errors');
            $response->send();
            wp_die();
        }
        $id = !empty($input['id']) ? (int) $input['id'] : null;

        $testimonial = new Testimonial(
            array(
                'id' => $id
            )
        );

        $id = $testimonial->save($input);

        $response = Response::get($this->response, $id, 'update_or_new_testimonial');
        $response->send();
        wp_die();
    }

    /**
     * Route for add Group Ajax request
     *
     * @return xml WP_Response
     */
    public function ajax_handle_update_or_new_group(){
        if(!empty($_POST['data'])){
            $prepare = $_REQUEST['data']['data'];
        }
        $input['id'] = isset($prepare['id']) ? $prepare['id'] : '';
        $input['name'] = isset($prepare['name']) ? $prepare['name'] : '';
        $input['description'] = isset($prepare['description']) ? $prepare['description'] : '';
        $input['custom_fields'] = array(
            'testimonials' => isset($prepare['testimonials_ids']) ? array_map('intval', $prepare['testimonials_ids']) : array(),
            'layout' => isset($prepare['layout']) ? $prepare['layout'] : '',
            'title' => isset($prepare['title']) ? $prepare['title'] : '',
            'custom_css' => isset($prepare['custom_css']) ? $prepare['custom_css'] : '',
            'form_custom_css' => isset($prepare['form_custom_css']) ? $prepare['form_custom_css'] : '',
            'order_by' => isset($prepare['order_by']) ? $prepare['order_by'] : '',
            'order_direction' => isset($prepare['order_direction']) ? $prepare['order_direction'] : '',
            'styles' => isset($prepare['styles']) ? $prepare['styles'] : array(),
            'panel' => isset($prepare['panel']) ? $prepare['panel'] : array(),
        );

        $validator = new Validator(Group::$model_name, self::$namespace);
        $validation = $validator->validate($input);

        if(count($validation->errors) > 0){
            $response = Response::get($this->response, $validation, 'errors');
            $response->send();
            wp_die();
        }
        $id = !empty($input['id']) ? (int) $input['id'] : null;

        $group = new Group(
            array(
                'id' => $id
            )
        );
        $id = $group->save($input);

        $response = Response::get($this->response, $id, 'update_or_new_group');
        $response->send();
        wp_die();
    }

    /**
     * Universal AJAX approve route
     */
    public function ajax_handle_publish_model(){
        $input = $_REQUEST['data'];
        $id = (int) $input['id'];

        if(isset($id)){
            $post = new Post(array('id' => $id));
            $post->publish();
            $response = Response::get($this->response, $id, 'publish_model');
        }else{
            $errors = new \WP_Error();
            $errors->add('delete-error', 'Id or model not defined');
            $response = Response::get($this->response, $errors, 'errors');
        }
        $response->send();
        exit();
    }

    /**
     * Universal AJAX delete route
     */
    public function ajax_handle_delete_model(){
        $input = $_REQUEST['data'];
        $id = (int) $input['id'];
        $model_name = $input['model_name'];

        if(isset($id) && isset($model_name)){
            $model_name = self::$namespace . '\App\Model\\' .  ucfirst($model_name);
            $post = new $model_name(array('id' => $id));
            $post->delete();
            $response = Response::get($this->response, $id, 'delete_model');
        }else{
            $errors = new \WP_Error();
            $errors->add('delete-error', 'Id or model not defined');
            $response = Response::get($this->response, $errors, 'errors');
        }
        $response->send();
        exit();
    }

    public function ajax_handle_get_group_preview_photo(){
        $input = $_REQUEST['data'];
        $id = (int) $input['id'];
        $size = $input['size'];

        $photo = get_the_post_thumbnail(
            $id,
            $size
        );

        $response = Response::get($this->response, json_encode($photo), 'group_preview_photo');
        $response->send();
        wp_die();
    }

    public function ajax_handle_get_group_testimonials_photos(){
        $input = $_REQUEST['data'];
        $ids = $input['ids'];
        $size = $input['size'];
        $result = array();

        $default_avatar_path = plugins_url('admin/img/default-avatar.png', dirname(__FILE__));
        $default_avatar = '<img src="' . $default_avatar_path . '" class="attachment-thumbnail" alt="" ?>';

        foreach($ids as $id){
            $photo = get_the_post_thumbnail(
                $id,
                $size
            );
            $result[] = $photo ?: $default_avatar;
        }
        $response = Response::get($this->response, json_encode($result), 'group_testimonials_photos');
        $response->send();
        wp_die();
    }

    public function ajax_handle_get_templates(){
        $templates = Template::get_templates();

        $response = Response::get($this->response, json_encode($templates), 'get_templates');
        $response->send();
        wp_die();
    }

    public function ajax_handle_get_taxonomy_terms(){
        $input = $_REQUEST['data'];
        $tax = $input['taxonomy'];
        $query = isset($input['query']) ? $input['query'] : '';
        $result = array();

        if($tax){
            $result = get_terms($tax, array(
                'hide_empty' => false,
                'name__like' => esc_attr($query)
            ));
        }

        $response = Response::get($this->response, json_encode($result), 'get_taxonomy_terms');
        $response->send();
        wp_die();

    }

    public function ajax_handle_get_tiny_mce(){
        ob_start();
        Editor::editor_html();
        $html = ob_get_contents();
        ob_end_clean();

        $response = Response::get($this->response, json_encode($html), 'init_tinymce');
        $response->send();
        wp_die();
    }
}