<?php
namespace ExcitedFree\App\Model;

use LooksAwesome\Common\ArrayUtil;
use LooksAwesome\Common\Post;
use ExcitedFree\App\Template;

class Group extends Post
{
    public static $model_name = 'group';
    public static $post_type = 'aet-group';

    public static $validation_rules = array(
        'name' => array(
            'rule' => 'required',
        ),
    );

    public static $fill_rules = array(
        'testimonials',
        'layout',
        'template',
        'styles' => array(
            'filter',
            'slider',
            'slider_autoplay',
            'slider_adaptive',

            'button_color',
            'button_hover_color',
            'button_text_color',
            'button_text_hover_color',

            'filter_bold',
            'filter_italic',

            'reveal',
            'hover',
            'bounce',

            'body_visible',
            'photo_visible',
            'name_visible',
            'divider_visible',
            'position_visible',
            'short_text_visible',
            'short_text_body_visible',
            'short_text_nubbin_visible',
            'source_visible',
            'rating_visible',
            'social_visible',

            'base_color',
            'card_base_color',
            'card_border_color',
            'card_divider_color',
            'card_shadow_color',
            'card_even_row_color',

            'name_bold',
            'name_italic',

            'position_bold',
            'position_italic',

            'short_text_bold',
            'short_text_italic',

            'contacts_bold',
            'contacts_italic',
        ),
        'panel' => array(
            'enabled',
            'theme',
            'color',
            'contacts_visible',
            'social_visible',
            'rating_visible',
        ),
    );

    public static $labels = array(
        'name' => 'Groups',
        'singular_name' => 'Group',
        'menu_name' => 'Groups',
        'name_admin_bar' => 'Group',
        'add_new' => 'Add Group',
        'add_new_item' => '+ Add new group',
        'new_item' => 'New Group',
        'edit_item' => 'Edit Group',
        'view_item' => 'View Group',
        'delete_item' => 'Delete Group',
        'all_items' => 'My Groups',
        'search_items' => 'Search Groups',
        'parent_item_colon' => 'Parent Groups:',
        'not_found' => 'No groups found.',
        'not_found_in_trash' => 'No groups found in Trash.',
    );

    public static $fields = array(
        'title',
        'editor',
        'excerpt',
    );

    protected static $defaults = array(
        'name' => '',
        'description' => '',
        'custom_fields' => array(
            'testimonials' => array(),
            'layout' => 'grid',
            'template' => 'magazine',
            'title' => '',
            'custom_css' => '',
            'order_by' => 'custom',
            'order_direction' => 'desc',
            'styles' => array(
                'grid' => array(),
                'grid_float' => array(),
                'single' => array(),
                'single_showcase' => array(),
                'widget' => array(),
            ),
            'panel' => array(
                'enabled' => '0',
                'theme' => 'clean',
                'contacts_visible' => '1',
                'social_visible' => '1',
                'rating_visible' => '0',
                'color' => '#E6E6E6',
                'animation' => 'push',
            ),
        ),
    );

    public static $layouts = array(
        'grid' => 'magazine',
        'grid_float' => 'twin_hearts',
        'single' => 'forrest',
        'single_showcase' => 'usual_suspects',
        'widget' => 'cookie',
    );

    protected static $form_blocks = array(
        'name',
        'position',
        'source',
        'source_url',
        'email',
        'phone',
        'rating',
        'short_text',
    );


    public function prepare_input($input)
    {
        $custom_fields = $this->getDefault('custom_fields');
        $data = array(
            'ID' => $this->id,
            'post_type' => static::$post_type,
            'post_status' => 'publish',
            'post_title' => $input['name'] ?: $this->post->post_title,
            'post_excerpt' => $input['description'],
            'custom_fields' => $this->assign_custom_fields($custom_fields, $input['custom_fields']),
        );

        return $data;
    }

    public function set_defaults($input, $defaults)
    {
        $result = array();
        foreach ($defaults as $key => $value) {
            if (gettype($value) == 'array' && ArrayUtil::is_assoc($value) && isset($input[$key])) {
                if ($key == 'blocks_order') {
                    $result[$key] = $input[$key];
                } else {
                    $result[$key] = $this->set_defaults($input[$key], $value);
                }
            } else {
                $result[$key] = isset($input[$key]) && $input[$key] !== '' ? $input[$key] : $defaults[$key];
            }
        }

        return $result;
    }

    public function assign_custom_fields($custom_fields, $input)
    {
        $data = array();
        foreach ($custom_fields as $key => $value) {
            if (!isset($input[$key])) {
                continue;
            }
            $field = $input[$key];
            if (gettype($value) == 'array' && ArrayUtil::is_assoc($field)) {
                if ($key == 'blocks_order') {
                    $data[$key] = $input[$key];
                } else {
                    $data[$key] = $this->assign_custom_fields($value, $field);
                }
            } else {
                $data[$key] = $field;
            }
        }

        return $data;
    }

    public function save($input = array())
    {
        $input = $this->set_defaults($input, static::getDefaults());
        $data = $this->prepare_input($input);

        if ($this->id) {
            /* Edit Post */
            $this->update_parents(
                $input['custom_fields']['testimonials'],
                Testimonial::$model_name.'s',
                static::$model_name.'s',
                $this->id
            );
            $wp_post = wp_update_post($data);
            $this->save_custom_fields($wp_post, $data['custom_fields'], true);
        } else {
            /* Insert New Post */
            $wp_post = wp_insert_post($data);
            $this->save_custom_fields($wp_post, $data['custom_fields']);
            $this->update_parents(
                $input['custom_fields']['testimonials'],
                Testimonial::$model_name.'s',
                static::$model_name.'s',
                $wp_post
            );
        }

        $this->after_save($input);

        return $wp_post;
    }

    public function after_save($input)
    {

    }

    /**
     * @return array
     */
    public static function getDefaults()
    {
        foreach (static::$layouts as $key => $value) {
            $template = Template::get_template($key);
            static::$defaults['custom_fields']['styles'][$key] = $template[$value]['styles'];
        }

        return static::$defaults;
    }

    public static function unsetDefault($styles, $group, $subgroup = null)
    {
        $def_instance = static::getDefaults();
        $defaults = $subgroup ? $def_instance['custom_fields'][$group][$subgroup] : $def_instance['custom_fields'][$group];
        $collection = isset($subgroup) ? $styles[$subgroup] : $styles;
        if (empty($collection)) {
            return;
        }
        foreach ($collection as $key => $value) {
            if (isset($collection[$key]) && !array_key_exists($key, $defaults)) {
                unset($collection[$key]);
            }
        }
        if (isset($subgroup)) {
            $styles[$subgroup] = $collection;
        } else {
            $styles = $collection;
        }

        return $styles;
    }

    public static function setDefault($styles, $group, $subgroup = null)
    {
        $def_instance = static::getDefaults();
        $defaults = $subgroup ? $def_instance['custom_fields'][$group][$subgroup] : $def_instance['custom_fields'][$group];
        $collection = isset($subgroup) ? $styles[$subgroup] : $styles;
        foreach ($defaults as $key => $value) {
            if (array_key_exists($key, $defaults) && !isset($collection[$key])) {
                $collection[$key] = $value;
            }
        }
        if (isset($subgroup)) {
            $styles[$subgroup] = $collection;
        } else {
            $styles = $collection;
        }

        return $styles;
    }

    /**
     * Set default value if field doesn't exist
     *
     * @param $post
     */
    public static function setDefaults($post)
    {
        if (isset($post->styles)) {
            $styles = $post->styles;
            foreach (static::$layouts as $key => $value) {
                $styles = static::unsetDefault($styles, 'styles', $key);
            }
            foreach (static::$layouts as $key => $value) {
                $styles = static::setDefault($styles, 'styles', $key);
            }
            $post->styles = $styles;
        }

        $panel = $post->panel;
        $panel = static::unsetDefault($panel, 'panel');
        $panel = static::setDefault($panel, 'panel');
        $post->panel = $panel;

        return $post;
    }

    public function getFormRules()
    {
        $rules = array();
        $post = self::setDefaults($this->post);
        foreach (self::$form_blocks as $block) {
            if (!isset($post->form[$block.'_visible']) || !isset($this->form[$block.'_required'])) {
                return false;
            }
            if ($this->form[$block.'_visible'] == '1' && $this->form[$block.'_required'] == '1') {
                $rules[$block] = array(
                    'rule' => 'required',
                );
            }
        }

        return $rules;
    }

    public static function getPhoto($id)
    {
        if (isset($id) && has_post_thumbnail($id)) {
            $foto_id = get_post_thumbnail_id($id);
            $foto = wp_get_attachment_image_src($foto_id, 'large', false);
            $photo = $foto[0];
        } else {
            $base = plugin_dir_url(realpath(__DIR__.'/..'));
            $photo = $base.'public/img/default-avatar.jpg';
        }

        return $photo;
    }

    /**
     * Cascade update for related testimonials
     * @param bool $force_delete
     */
    public function delete($force_delete = true)
    {
        if (isset($this->post->testimonials)) {
            foreach ($this->post->testimonials as $id) {
                $testimonial = Testimonial::find($id);
                $groups = $testimonial->groups ?: array();
                if (($key = array_search($this->id, $groups)) !== false) {
                    unset($groups[$key]);
                    update_post_meta($testimonial->ID, 'groups', $groups);
                }
            }
        }
        wp_delete_post($this->id, $force_delete);
    }
} 