<?php
namespace ExcitedFree\App;

use ExcitedFree\App\Model\Testimonial;
use ExcitedFree\App\Model\Group;
use LooksAwesome\Common\Render;

class Shortcode
{

    private $layout = '';
    private $group = null;
    public static $ad = '<!-- Excited Testimonials by Looks Awesome: http://testimonial.looks-awesome.com -->';
    private static $plugin_name;
    private static $mode;

    public function __construct()
    {
        $env = null;
        if (file_exists(plugin_dir_path(__DIR__) . 'env.json')) {
            $env = json_decode(file_get_contents(plugin_dir_path(__DIR__) . 'env.json'), true);
        }

        self::$plugin_name = \LA_Testimonials_Free::$plugin['name'];
        self::$mode = $env['mode'] ?: 'dev';
        $shortcode = \LA_Testimonials_Free::$plugin['shortcode'];
        add_shortcode($shortcode, array($this, 'render')); // set Awesome Group shortcode
    }

    /**
     * Decode group post meta field shortcode
     *
     * @param $id
     * @param bool $preview
     * @return array
     */

    public function set_group($id, $preview = false)
    {
        $post = Group::find($id);
        return $this->group = $this->decode($post, $preview);
    }

    /**
     * @param null $layout
     */
    public function set_layout($layout = null)
    {
        $layout = isset($layout) ? $layout : $this->group->layout;
        $s = DIRECTORY_SEPARATOR;
        $this->layout = WP_PLUGIN_DIR . $s . self::$plugin_name . $s . 'public' . $s . 'partials' . $s . $layout . '.php';
    }


    public function enqueue_all(){
        if (self::$mode == 'dev') {
            wp_enqueue_style(self::$plugin_name . '-shortcode');
            wp_enqueue_style(self::$plugin_name . '-panel');
            wp_enqueue_style(self::$plugin_name . '-slick-carousel-theme');
            wp_enqueue_style(self::$plugin_name . '-animation');
            wp_enqueue_style('slick-carousel');
            wp_enqueue_style('awesome-panel');

            wp_enqueue_script('slick-carousel');
            wp_enqueue_script('awesome-ajax');
            wp_enqueue_script('awesome-filter');
            wp_enqueue_script('awesome-viewport');
            wp_enqueue_script('awesome-panel');
            wp_enqueue_script('awesome-util');
            wp_enqueue_script(self::$plugin_name . '-shortcode');
        } else {
            wp_enqueue_style(self::$plugin_name . '-all');
            wp_enqueue_script(self::$plugin_name . '-all');
        }

        wp_enqueue_style(self::$plugin_name.'-font-awesome');
        wp_enqueue_script('wp-ajax-response');
        wp_enqueue_script('google-recaptcha');
        wp_enqueue_script('wp-ajax-response');
    }

    /**
     * @return string
     */
    public function get_layout_html()
    {
        $view = new Render($this->layout, $this->group); // template, data
        return $view->render();
    }

    public static function hex_to_rgb($hex)
    {
        if (strlen($hex) == 0) {
            return '';
        }
        $hex = str_replace('#', '', $hex);
        if (strlen($hex) == 3) {
            $r = hexdec(substr($hex, 0, 1) . substr($hex, 0, 1));
            $g = hexdec(substr($hex, 1, 1) . substr($hex, 1, 1));
            $b = hexdec(substr($hex, 2, 1) . substr($hex, 2, 1));
        }
        if (strlen($hex) == 6) {
            $r = hexdec(substr($hex, 0, 2));
            $g = hexdec(substr($hex, 2, 2));
            $b = hexdec(substr($hex, 4, 2));
        }
        $rgb = array($r, $g, $b);
        return implode(', ', $rgb);
    }

    /**
     * @param $post
     * @param $preview
     * @return WP_Post
     */
    public function decode($post, $preview)
    {

        if (null == $post) {
            return false;
        }


        $post = Group::setDefaults($post);
        $post->filter_terms = array();
        $testimonials_ids = $post->testimonials;
        $option_order_by = esc_html($post->order_by);
        $option_order_direction = esc_html($post->order_direction);

        if ($preview) {
            $post->preview = true;
        }

        if (!empty($testimonials_ids)) {
            $post->testimonials = array();
            $post->filter_terms = wp_get_object_terms(
                $testimonials_ids,
                Testimonial::getTaxonomy()
            );

            if($option_order_by == 'custom'){
                $order_by = 'post__in';
            }
            if($option_order_by == 'publish_date'){
                $order_by = 'date';
            }
            if($option_order_by == 'random'){
                $order_by = 'rand';
            }

            $testimonials = Testimonial::where(
                array(
                    'post_type' => Testimonial::$post_type,
                    'post__in' => $testimonials_ids,
                    'orderby' => $order_by,
                    'no_found_rows' => true,
                    'numberposts' => -1,
                    'posts_per_page' => -1,
                    'order' => $option_order_direction
                )
            );

            foreach ($testimonials as $testimonial) {
                $post->testimonials[] = $testimonial;
            }
        }

        if ($preview) {
            $post->testimonials = array();

            $post->testimonials[0] = new \stdClass;
            $post->testimonials[0]->ID = 0;
            $post->testimonials[0]->post_title = 'John Doe';
            $post->testimonials[0]->position = 'Big Brother';
            $post->testimonials[0]->post_excerpt = 'Lorem ipsum dolor sit amet, eum ea labores vivendo. Sea te porro consul, ad qui habeo persius repudiandae. Duo eu persius detracto mediocritatem.';
            $post->testimonials[0]->email = 'john@acme.com';
            $post->testimonials[0]->phone = '555-5555-5555';
            $post->testimonials[0]->skype = 'john_doe';
            $post->testimonials[0]->source_name = 'Source';
            $post->testimonials[0]->source_url = 'http://domain.com';
            $post->testimonials[0]->custom_url = '';
            $post->testimonials[0]->location = 'Madagascar';
            $post->testimonials[0]->profile = '';
            $post->testimonials[0]->facebook = 'http://facebook.com';
            $post->testimonials[0]->twitter = 'http://facebook.com';
            $post->testimonials[0]->linkedin = 'http://facebook.com';
            $post->testimonials[0]->google = 'http://google.com';
            $post->testimonials[0]->instagram = 'http://instagram.com';
            $post->testimonials[0]->youtube = 'http://youtube.com';
            $post->testimonials[0]->pinterest = '';
            $post->testimonials[0]->vimeo = '';
            $post->testimonials[0]->dribble = '';
            $post->testimonials[0]->flickr = '';
            $post->testimonials[0]->github = '';
            $post->testimonials[0]->rating = 80;
        }

        return $post;
    }

    /**
     * Render shortcode preview for admin area
     *
     * @param $post
     * @param $layout
     * @return mixed
     */
    public function preview($post, $layout)
    {
        if ($this->group == null) {
            $this->group = $this->decode($post, true);
        }
        $this->set_layout($layout);
        $html = $this->get_layout_html();
        return $html;
    }


    /**
     * Render shortcode on frontend
     *
     * @param $atts
     * @return mixed
     */
    public function render($atts)
    {
        // enqueue scripts & styles only if shortcode included on page
        $args = shortcode_atts(array(
            'id' => ''
        ), $atts);

        if ($this->set_group($args['id'])) {
            $this->set_layout();
            $this->enqueue_all();
            return $this->get_layout_html();
        } else {
            return false;
        }
    }

    public static function get_blocks_order_clean($blocks, $styles){
        $result = array();
        foreach($blocks as $key => $value){
            if(is_array($value)){
                $result[$key] = static::get_blocks_order_clean($value, $styles);
            }else {
                if($styles[$value . '_visible'] == '1'){
                    $result[$key] = $value;
                }
            }
        }
        return $result;
    }

    public static function get_blocks($blocks_order){
        $result = array();
        foreach ($blocks_order as $block) {
            if(is_array($block)){
                $result = array_merge($result, static::get_blocks($block));
            }else {
                array_push($result, $block);
            }
        }
        return $result;
    }

    public static function include_blocks($atts, $level = 1){
        extract($atts);
        foreach ($blocks_order as $block) {
            if(gettype($block) === 'array'){
                $atts['blocks_order'] = $block;
                echo '<div class="sortable-box level-' . ++$level . '">';
                self::include_blocks($atts, $level);
                echo '</div>';
            }else {
                $atts['block'] = $block;
                static::include_block($atts);
            }
        }
    }

    /**
     * Include block for layout
     */
    public static function include_block($atts){
        $defaults = array(
            'force_visible' => false
        );
        $atts = $atts + $defaults;
        extract($atts);
        $base = plugin_dir_path(dirname(__FILE__));
        if(isset($styles[$block . '_visible'])){
            $visible = $styles[$block . '_visible'];
            $common_path = $base . "public/partials/blocks/$block.php";
            $special_path = $base . "public/partials/blocks/$layout/$block.php";
            if ($visible == '1' || $preview || $force_visible) {
                if (file_exists($special_path)) {
                    include($special_path);
                    return;
                }
                if (file_exists($common_path)) {
                    include($common_path);
                    return;
                }
            }
        }
    }

    public static function get_image_sizes($size = ''){
        global $_wp_additional_image_sizes;

        $sizes = array();
        $get_intermediate_image_sizes = get_intermediate_image_sizes();

        // Create the full array with sizes and crop info
        foreach ($get_intermediate_image_sizes as $_size) {
            if (in_array($_size, array('thumbnail', 'medium', 'large'))) {
                $sizes[$_size][0] = get_option($_size . '_size_w');
                $sizes[$_size][1] = get_option($_size . '_size_h');
                //$sizes[ $_size ][2] = (bool) get_option( $_size . '_crop' );
                $sizes[$_size][2] = true;
            } elseif (isset($_wp_additional_image_sizes[$_size])) {
                $sizes[$_size] = array(
                    $_wp_additional_image_sizes[$_size]['width'],
                    $_wp_additional_image_sizes[$_size]['height'],
                    $_wp_additional_image_sizes[$_size]['crop']
                );

            }

        }

        // Get only 1 size if found
        if ($size) {

            if (isset($sizes[$size])) {
                return $sizes[$size];
            } else {
                return false;
            }

        }

        return $sizes;
    }

    /**
     * Get size in pixels or %
     * @param $raw
     * @return string
     */
    public static function get_size($raw)
    {
        if (substr($raw, -1) == '%') {
            return $raw;
        }
        if (substr($raw, -2) == 'px') {
            return $raw;
        }
        $result = preg_replace('/[^0-9]/', '', $raw);
        return $result . 'px';;
    }
}