<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://looks-awesome.com
 * @since             1.0.0
 * @package           A_Group_Showcase_Free
 *
 * @wordpress-plugin
 * Plugin Name:       Excited Testimonials (Free)
 * Plugin URI:        http://testimonial.looks-awesome.com/
 * Description:       Awesome Testimonials plugin (free version)
 * Version:           1.0.5
 * Author:            Looks Awesome
 * Author URI:        http://looks-awesome.com
 * Text Domain:       excited-testimonials-showcase
 * Domain Path:       /languages
 */

// If this file is called directly, abort.

if ( ! defined( 'WPINC' ) ) {
	die;
}

class LA_Testimonials_Free{
    protected $activator;
    protected $deactivator;
    public static $plugin = array(
        'name' => 'excited-testimonials-showcase',
        'shortcode' => 'a-excited-testimonials',
        'label' => 'Testimonials',
        'prefix' => 'aetfree',
        'demo_site' => 'http://testimonial.looks-awesome.com',
        'version' => '1.0.5',
        'require_php' => '5.3.0',
        'namespace' => 'ExcitedFree',
        'base' => '',
        'base_url' => '',
        'mode' => ''
    );

    public static function init(){
        $activator = new LA_Testimonials_App_ActivatorFree(self::$plugin);
        $deactivator = new LA_Testimonials_App_DeactivatorFree(self::$plugin);

        add_action( 'admin_init', array( 'LA_Testimonials_App_ActivatorFree', 'check_version' ) );
        if(!call_user_func(array($activator, 'compatible_version'))){
            return;
        }
        self::$plugin['base'] = plugin_dir_path(__FILE__);
        self::$plugin['base_url'] = plugin_dir_url(__FILE__);
        $env = null;
        if(file_exists(self::$plugin['base'] . 'env.json')){
            $env = json_decode(file_get_contents(self::$plugin['base'] . 'env.json'), true);
        }
        self::$plugin['mode'] = $env['mode'] ? $env['mode'] : 'dev';

        new LA_Testimonials_App_PluginFree(self::$plugin);
    }

    public static function activate(){
        LA_Testimonials_App_ActivatorFree::activate();
    }

    public static function deactivate(){
        LA_Testimonials_App_DeactivatorFree::deactivate();
    }

    public static function autoloader($class){
        $base = plugin_dir_path(__FILE__);
        if(!class_exists('LA_Testimonials_App_AutoloaderFree')){
            include_once $base . 'includes/AutoloaderFree.php';
        }
        LA_Testimonials_App_AutoloaderFree::load($class, self::$plugin, $base);
    }
}

register_activation_hook( __FILE__, 'LA_Testimonials_Free::activate' );
register_deactivation_hook( __FILE__, 'LA_Testimonials_Free::deactivate' );

/**
 * Plugin Autoload
 */

// composer autoload
require __DIR__ . '/includes/vendor/autoload.php';

// looks awesome autoload
spl_autoload_register( 'LA_Testimonials_Free::autoloader' );

LA_Testimonials_Free::init();